/// @dev importing packages required
const assert = require('assert');
const ethers = require('ethers');
const ganache = require('ganache-core');
const { parseTx } = require('../helpers');

/// @dev when you make this true, the parseTx helper will output transaction gas consumption and logs
const DEBUG_MODE = false;

/// @dev initialising development blockchain
const provider = new ethers.providers.Web3Provider(
  ganache.provider({ gasLimit: 8000000 })
);

/// @dev importing build file
const erc20JSON = require('../build/ERC20_ERC20.json');

/// @dev initialize global variables
let accounts, erc20Instance;

/// @dev this is a test case collection
describe('Ganache Setup', async () => {
  /// @dev this is a test case. You first fetch the present state, and compare it with an expectation. If it satisfies the expectation, then test case passes else an error is thrown.
  it('initiates ganache and generates a bunch of demo accounts', async () => {
    /// @dev for example in this test case we are fetching accounts array.
    accounts = await provider.listAccounts();

    /// @dev then we have our expection that accounts array should be at least having 1 accounts
    assert.ok(
      accounts.length >= 1,
      'atleast 2 accounts should be present in the array'
    );
  });
});

/// @dev this is another test case collection
describe('ERC 20 Contract', () => {
  /// @dev describe under another describe is a sub test case collection
  describe('ERC 20 Setup', async () => {
    /// @dev this is first test case of this collection
    it('deploys ERC 20 contract from first account with initial storage: Hello World', async () => {
      /// @dev you create a contract factory for deploying contract. Refer to ethers.js documentation at https://docs.ethers.io/ethers.js/html/
      const ERC20ContractFactory = new ethers.ContractFactory(
        erc20JSON.abi,
        erc20JSON.evm.bytecode.object,
        provider.getSigner(accounts[0])
      );
      erc20Instance = await ERC20ContractFactory.deploy();

      assert.ok(erc20Instance.address, 'conract address should be present');
    });

    /// @dev this is second test case of this collection
    it('checks erc20 balance of deployer if its postitve', async () => {
      /// @dev you access the value at storage with ethers.js library of our custom contract method called getValue defined in contracts/ERC20.sol
      const balance = await erc20Instance.functions.balanceOf(accounts[0]);

      console.log(balance);

      /// @dev then you compare it with your expectation value
      assert.ok(balance.gte(0), 'deployer should receive balance');
    });
  });
});
